#include "vrayblinnmtl.h"
#include "pb2template_generator.h"

/*===========================================================================*\
 |	Class Descriptor
\*===========================================================================*/

class SkeletonMaterialClassDesc: public ClassDesc2
#if GET_MAX_RELEASE(VERSION_3DSMAX) >= 6000
, public IMtlRender_Compatibility_MtlBase 
#endif
{
	HIMAGELIST imageList;
public:
	int IsPublic() { return TRUE; }
	void* Create(BOOL loading) { return new SkeletonMaterial(loading); }
	const TCHAR *	ClassName() { return STR_CLASSNAME; }
	SClass_ID SuperClassID() { return MATERIAL_CLASS_ID; }
	Class_ID ClassID() { return MTL_CLASSID; }
	const TCHAR* Category() { return _T("");  }

	// Hardwired name, used by MAX Script as unique identifier
	const TCHAR* InternalName() { return STR_CLASSNAME; }
	HINSTANCE HInstance() { return hInstance; }

	SkeletonMaterialClassDesc(void) {
		imageList=NULL;
#if GET_MAX_RELEASE(VERSION_3DSMAX) >= 6000
		IMtlRender_Compatibility_MtlBase::Init(*this);
#endif
	}

	~SkeletonMaterialClassDesc(void) {
		if (imageList) ImageList_Destroy(imageList);
		imageList=NULL;
	}

#if GET_MAX_RELEASE(VERSION_3DSMAX) >= 6000
	// From IMtlRender_Compatibility_MtlBase
	bool IsCompatibleWithRenderer(ClassDesc& rendererClassDesc) {
		if (rendererClassDesc.ClassID()!=VRENDER_CLASS_ID) return false;
		return true;
	}
	bool GetCustomMtlBrowserIcon(HIMAGELIST& hImageList, int& inactiveIndex, int& activeIndex, int& disabledIndex) {
		if (!imageList) {
			HBITMAP bmp=LoadBitmap(hInstance, MAKEINTRESOURCE(bm_MTL));
			HBITMAP mask=LoadBitmap(hInstance, MAKEINTRESOURCE(bm_MTLMASK));

			imageList=ImageList_Create(11, 11, ILC_COLOR24 | ILC_MASK, 5, 0);
			int index=ImageList_Add(imageList, bmp, mask);
			if (index==-1) return false;
		}

		if (!imageList) return false;

		hImageList=imageList;
		inactiveIndex=0;
		activeIndex=1;
		disabledIndex=2;
		return true;
	}
#endif
};

static SkeletonMaterialClassDesc SkelMtlCD;
ClassDesc* GetSkeletonMtlDesc() {return &SkelMtlCD;}

/*===========================================================================*\
 |	Basic implimentation of a dialog handler
\*===========================================================================*/

/*===========================================================================*\
 |	Paramblock2 Descriptor
\*===========================================================================*/

static int numID=100;
int ctrlID(void) { return numID++; }

static ParamBlockDesc2 smtl_param_blk ( mtl_params, _T("SkeletonMaterial parameters"),  0, &SkelMtlCD, P_AUTO_CONSTRUCT + P_AUTO_UI, 0, 
	//rollout
	IDD_SKELETON_MATERIAL, IDS_PARAMETERS, 0, 0, NULL, 
	// params
	mtl_diffuse, _T("diffuse"), TYPE_RGBA, P_ANIMATABLE, 0,
		p_default, Color(0.5f, 0.5f, 0.5f),
		p_ui, TYPE_COLORSWATCH, ctrlID(),
	end,
	mtl_color, _T("reflection"), TYPE_RGBA, P_ANIMATABLE, 0,
		p_default, Color(0.0f, 0.0f, 0.0f),
		p_ui, TYPE_COLORSWATCH, ctrlID(),
	end,
	mtl_glossiness, _T("glossiness"), TYPE_FLOAT, P_ANIMATABLE, 0,
		p_default, 0.5f,
		p_range, 0.0f, 1.0f,
		p_ui, TYPE_SPINNER, EDITTYPE_FLOAT, ctrlID(), ctrlID(), 0.1f,
	end,
	mtl_opacity, _T("opacity"), TYPE_RGBA, P_ANIMATABLE, 0,
		p_default, Color(1.0f, 1.0f, 1.0f),
		p_ui, TYPE_COLORSWATCH, ctrlID(),
	end,
end
);

/*===========================================================================*\
 |	UI stuff
\*===========================================================================*/

class SkelMtlDlgProc : public ParamMap2UserDlgProc {
public:
	IParamMap *pmap;
	SkeletonMaterial *sm;

	SkelMtlDlgProc(void) { sm = NULL; }
	INT_PTR DlgProc(TimeValue t, IParamMap2 *map, HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
		int id = LOWORD(wParam);
		switch (msg) 
		{
			case WM_INITDIALOG:
				break;
			case WM_DESTROY:
				break;
			case WM_COMMAND: {
				break;
			}
		}
		return FALSE;
	}
	void DeleteThis() {}
	void SetThing(ReferenceTarget *m) { sm = (SkeletonMaterial*)m; }
};

static SkelMtlDlgProc dlgProc;
static Pb2TemplateGenerator templateGenerator;

class SkelMtlParamDlg: public ParamDlg {
public:
	SkeletonMaterial *mtl;
	IMtlParams *ip;
	IParamMap2 *pmap;
	
	SkelMtlParamDlg(SkeletonMaterial *m, HWND hWnd, IMtlParams *i) {
		mtl=m;
		ip=i;
		
		DLGTEMPLATE* tmp=templateGenerator.GenerateTemplate(mtl->pblock, STR_DLGTITLE, 217);
		pmap=CreateMParamMap2(mtl->pblock, ip, hInstance, hWnd, NULL, NULL, tmp, STR_DLGTITLE, 0, &dlgProc);
		templateGenerator.ReleaseDlgTemplate(tmp);
	}

	void DeleteThis(void) {
		if (pmap) DestroyMParamMap2(pmap);
		pmap=NULL;
		delete this;
	}
	
	Class_ID ClassID(void) { return MTL_CLASSID; }
	void SetThing(ReferenceTarget *m) { mtl=(SkeletonMaterial*) m; pmap->SetParamBlock(mtl->pblock); }
	ReferenceTarget* GetThing(void) { return mtl; }
	void SetTime(TimeValue t) {}
	void ReloadDialog(void) {}
	void ActivateDlg(BOOL onOff) {}
};

/*===========================================================================*\
 |	Constructor and Reset systems
 |  Ask the ClassDesc2 to make the AUTO_CONSTRUCT paramblocks and wire them in
\*===========================================================================*/

void SkeletonMaterial::Reset() {
	ivalid.SetEmpty();
	SkelMtlCD.Reset(this);
}

SkeletonMaterial::SkeletonMaterial(BOOL loading) {
	pblock=NULL;
	ivalid.SetEmpty();
	SkelMtlCD.MakeAutoParamBlocks(this);	// make and intialize paramblock2
}

ParamDlg* SkeletonMaterial::CreateParamDlg(HWND hwMtlEdit, IMtlParams *imp) {
	return new SkelMtlParamDlg(this, hwMtlEdit, imp);
	/*
	IAutoMParamDlg* masterDlg = SkelMtlCD.CreateParamDlgs(hwMtlEdit, imp, this);
	smtl_param_blk.SetUserDlgProc(new SkelMtlDlgProc(this));
	return masterDlg;
	*/
}

BOOL SkeletonMaterial::SetDlgThing(ParamDlg* dlg) {
	return FALSE;
}

Interval SkeletonMaterial::Validity(TimeValue t) {
	Interval temp=FOREVER;
	Update(t, temp);
	return ivalid;
}

/*===========================================================================*\
 |	Subanim & References support
\*===========================================================================*/

RefTargetHandle SkeletonMaterial::GetReference(int i) {
	if (i==0) return pblock;
	return NULL;
}

void SkeletonMaterial::SetReference(int i, RefTargetHandle rtarg) {
	if (i==0) pblock=(IParamBlock2*) rtarg;
}

TSTR SkeletonMaterial::SubAnimName(int i) {
	if (i==0) return _T("Parameters");
	return _T("");
}

Animatable* SkeletonMaterial::SubAnim(int i) {
	if (i==0) return pblock;
	return NULL;
}

RefResult SkeletonMaterial::NotifyRefChanged(Interval changeInt, RefTargetHandle hTarget, PartID& partID, RefMessage message ) {
	switch (message) {
		case REFMSG_CHANGE:
			ivalid.SetEmpty();
			if (hTarget==pblock) {
				ParamID changing_param = pblock->LastNotifyParamID();
				smtl_param_blk.InvalidateUI(changing_param);
			}
			break;
	}
	return(REF_SUCCEED);
}

/*===========================================================================*\
 |	Standard IO
\*===========================================================================*/

#define MTL_HDR_CHUNK 0x4000

IOResult SkeletonMaterial::Save(ISave *isave) { 
	IOResult res;
	isave->BeginChunk(MTL_HDR_CHUNK);
	res = MtlBase::Save(isave);
	if (res!=IO_OK) return res;
	isave->EndChunk();
	return IO_OK;
}	

IOResult SkeletonMaterial::Load(ILoad *iload) { 
	IOResult res;
	int id;
	while (IO_OK==(res=iload->OpenChunk())) {
		switch(id = iload->CurChunkID())  {
			case MTL_HDR_CHUNK:
				res = MtlBase::Load(iload);
				break;
		}
		iload->CloseChunk();
		if (res!=IO_OK) return res;
	}
	return IO_OK;
}

/*===========================================================================*\
 |	Updating and cloning
\*===========================================================================*/

RefTargetHandle SkeletonMaterial::Clone(RemapDir &remap) {
	SkeletonMaterial *mnew = new SkeletonMaterial(FALSE);
	*((MtlBase*)mnew) = *((MtlBase*)this); 
	BaseClone(this, mnew, remap);
	mnew->ReplaceReference(0, remap.CloneRef(pblock));
	mnew->ivalid.SetEmpty();	
	return (RefTargetHandle) mnew;
}

void SkeletonMaterial::NotifyChanged() {
	NotifyDependents(FOREVER, PART_ALL, REFMSG_CHANGE);
}

void SkeletonMaterial::Update(TimeValue t, Interval& valid) {
	if (!ivalid.InInterval(t)) {
		ivalid.SetInfinite();

		pblock->GetValue(mtl_glossiness, t, glossiness, ivalid);
		pblock->GetValue(mtl_color, t, reflect, ivalid);
		pblock->GetValue(mtl_diffuse, t, diffuse, ivalid);
		pblock->GetValue(mtl_opacity, t, opacity, ivalid);
	}

	valid &= ivalid;
}

void SkeletonMaterial::renderBegin(TimeValue t, VR::VRayRenderer *vray) {
	const VR::VRaySequenceData &sdata=vray->getSequenceData();
	bsdfPool.init(sdata.maxRenderThreads);
}

void SkeletonMaterial::renderEnd(VR::VRayRenderer *vray) {
	bsdfPool.freeMem();
	renderChannels.freeMem();
}
